function [ims,dirs]=directimg2(im,varargin)
global dirim2_data
if ~isstruct(dirim2_data)
  dirim2_data.ok=0;
end

if ~isempty(varargin) & isstruct(varargin{1})
  kv = varargin{1};
else
  kv = getopt('+ pwr=1 lambda4=[] sigperp=[] siglong=[] ndir=8 onlight=0 ondark=0 softmax=0',varargin);
end

[Y X]=size(im);
ims=zeros([Y X kv.ndir]);

pwrs=[ 1.0000 1.0500 1.1000 1.1500 1.2000 1.2500 1.3000 1.3500 1.4000 1.4500 1.5000 1.5500 1.6000 1.6500 1.7000 1.7500 1.8000 1.8500 1.9000 1.9500 2.0000 ];
sclf=[ 0.4313 0.4814 0.5167 0.5352 0.5453 0.5509 0.5539 0.5553 0.5556 0.5553 0.5545 0.5534 0.5522 0.5508 0.5494 0.5479 0.5465 0.5450 0.5436 0.5422 0.5409 ];

if isempty(kv.pwr)
  kv.pwr=1;
end
if kv.pwr<min(pwrs)
  scl=sclf(1);
  fprintf(1,'Warning: directimg2: Gabor patch not balanced.\n');
elseif kv.pwr>max(pwrs)
  scl=sclf(end);
  fprintf(1,'Warning: directimg2: Gabor patch not balanced.\n');
else
  scl = interp1(pwrs,sclf,kv.pwr,'cubic');
end

if isempty(kv.lambda4)
  kv.lambda4 = kv.sigperp * scl;
end
if isempty(kv.sigperp)
  kv.sigperp = kv.lambda4 / scl;
end
if isempty(kv.siglong)
  kv.siglong = kv.sigperp;
end

lambda = 4*kv.lambda4;
omega = 2*pi/lambda;

sigmax=max(kv.sigperp,kv.siglong);
patsize = ceil(2*sigmax);

S=size(im);
if dirim2_data.ok
  if dirim2_data.ndir==kv.ndir & dirim2_data.patsize==patsize & ...
	dirim2_data.sigperp==kv.sigperp & dirim2_data.siglong==kv.siglong & ...
	dirim2_data.lambda4==kv.lambda4 & ...
	all(dirim2_data.S==S)
    ; % Nothing to be done
  else
    dirim2_data.ok = 0;
  end
end
if ~dirim2_data.ok
  dirim2_data.ndir = kv.ndir;
  dirim2_data.patsize = patsize;
  dirim2_data.sigperp = kv.sigperp;
  dirim2_data.siglong = kv.siglong;
  dirim2_data.lambda4 = kv.lambda4;
  dirim2_data.S = S;
  dirim2_data.patz={};
  
  xx=[0:S(2)-1]; xx(xx>=S(2)/2)=xx(xx>=S(2)/2)-S(2);
  yy=[0:S(1)-1]'; yy(yy>=S(1)/2)=yy(yy>=S(1)/2)-S(1);
  xx=repmat(xx,[S(1) 1]);
  yy=repmat(yy,[1 S(2)]);
  
  %xx=repmat([-patsize:patsize],[2*patsize+1 1]);
  %yy=xx';
  for n=1:kv.ndir
    phi = pi*n/kv.ndir;
    xi=cos(phi)*xx-sin(phi)*yy;
    eta=sin(phi)*xx+cos(phi)*yy;
    pat=exp(-.5*xi.^2/kv.sigperp^2-.5*eta.^2/kv.siglong.^2) .* cos(omega*xi);
    %% balance:
    %s1=sum(pat(pat>0));
    %s2=sum(pat(pat<0));
    %scl=sqrt(-s1/s2);
    %pat(pat>0)=pat(pat>0)/scl;
    %pat(pat<0)=pat(pat<0)/scl;
    %% done.
    %dirim2_data.patz{n} = conj(fft2(pat,S(1),S(2)));
    dirim2_data.patz{n} = conj(fft2(pat));
  end
  dirim2_data.ok = 1;
end

imz=fft2(im);
for n=1:kv.ndir
  %ims(:,:,n) = conv2(im,pat,'same');
  ims(:,:,n) = real(ifft2(imz.*dirim2_data.patz{n}));
end

edge=ceil(1.5*sigmax);
ims(1:edge,:,:)=0;
ims(end-edge+1:end,:,:)=0;
ims(:,1:edge,:)=0;
ims(:,end-edge+1:end,:,:)=0;

phi = pi * [1:kv.ndir]/kv.ndir;

if kv.onlight
  if kv.ondark
    ims=abs(ims);
  else
    ims=-ims;
  end
else
  % Default is 'ondark'
end

if nargout>=2
  if kv.softmax
    ims(ims<0)=0;
    smax = complex(zeros(Y,X));
    for n=1:kv.ndir
      smax = smax + ims(:,:,n).^2 * exp(2*i*phi(n));
    end
    ims = sqrt(abs(smax));
    dirs = angle(smax)/2;
  else
    [ims,dirs]=max(ims,[],3);
    dirs=phi(dirs);
  end
end
